/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//AwakeRemoteException
//Copyright (c) Kawan Softwares S.A.S, 2012
//
//Last Updates: 
// 29 sept. 2010 15:56:54 Kawan Softwares S.A.S
// 05/12/11 15:15 NDP : AwakeRemoteException: clean Javadoc
// 19/03/12 15:55 NDP : AwakeRemoteException: now extends IOException

package org.awakefw.commons.api.client;

import java.io.IOException;


/**
 * Signals that an Exception has been thrown by a remote Awake method on the server.
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 */
public class RemoteException extends IOException {

    private static final long serialVersionUID = -5604183624284785327L;

    private String remoteStackTrace = null;

    /**
     * Constructs a new RemoteException with the specified cause and a
     * detail message of <tt>(cause==null ? null : cause.toString())</tt> (which
     * typically contains the class and detail message of <tt>cause</tt>). This
     * constructor is useful for exceptions that are little more than wrappers
     * for other throwables (for example,
     * {@link java.security.PrivilegedActionException}).
     * 
     * @param message
     *            The detail message (which is saved for later retrieval by the
     *            {@link #getMessage()} method)
     * 
     * @param cause
     *            The cause (which is saved for later retrieval by the
     *            {@link #getCause()} method). (A null value is permitted, and
     *            indicates that the cause is nonexistent or unknown.)
     * 
     * @param remoteStackTrace
     *            the remote stack trace as string (null if none)
     */
    public RemoteException(String message, Throwable cause,
	    String remoteStackTrace) {
	super(message, cause);
	this.remoteStackTrace = remoteStackTrace;
    }

    /**
     * Returns the remote stack trace as a display string.
     * 
     * @return the remote Stack Trace as a display string
     */
    public String getRemoteStackTrace() {
	return this.remoteStackTrace;
    }

}
