/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//Sha1.java
//Copyright (c) Kawan Softwares S.A.S, 2012
//
//Last Updates: 
// 14 oct. 2005 17:01:13 Nicolas de Pomereu
// 31/12/05 13:10 NDP - fix bug on order parameter in getInstance
// 06/01/06 17:30 NDP - add File Hash
// 17/09/08 10:45 NDP - Silent close of input stream in finally
// 16/02/12 15:20 NDP : Sha1: move to org.awakefw.commons.api.server.util & clean Javadoc. 
// 21/02/12 19:15 NDP : Sha1: clean code
// 25/07/12 17:35 NDP : Sha1: no more reference to "SUN" provider ==> fails on Android
// 28/11/12 12:05 NDP : Sha1: no uses org.awakefw.file.api.util.Sha1Util with delegation to keep
//			one core class
// 19/01/13 11:55 NDP : Sha1: constructor is now on top of class

package org.awakefw.commons.api.server.util;

import java.io.File;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import org.awakefw.file.api.util.Sha1Util;

/**
 * SHA-1 hash functions on strings and files with default provider.
 * 
 * @author Nicolas de Pomereu
 * 
 */
public class Sha1 {


    private Sha1Util sha1Util = null;
    
    /**
     * Constructor.
     */
    public Sha1() {
	sha1Util = new Sha1Util();
    }

    
    /**
     * Computes the SHA-1 hash code of a byte array as an hex string.
     * 
     * @param b
     *            the bytes to hash
     * @return the hash value
     * @exception NoSuchAlgorithmException
     *                if the algorithm is not available from the provide
     * @exception NoSuchProviderException
     *                if the provider is not available in the environment
     */
    public byte[] getHash(byte[] b) throws NoSuchAlgorithmException,
	    NoSuchProviderException {
	return sha1Util.getHash(b);
    }

    /**
     * 
     * Computes the hash code of a byte array as an hex string.
     * 
     * @param b
     *            the bytes to hash <br>
     * @return the hash value in hex String
     * @exception NoSuchAlgorithmException
     *                if the algorithm is not available from the provider
     * @exception NoSuchProviderException
     *                if the provider is not available in the environment
     */
    public String getHexHash(byte[] b) throws NoSuchAlgorithmException,
	    NoSuchProviderException {
	return sha1Util.getHexHash(b);
    }

    /**
     * Gets the a file hash value.
     * 
     * @param file the file to hash
     *            
     * @return the file hash as bytes
     * 
     * @exception IOException
     *                an I/O error occurred
     * @exception NoSuchAlgorithmException
     *                hash algorithm not found
     * @exception NoSuchProviderException
     *                hash provider not found
     */
    public byte[] getFileHash(File file) throws IOException,
	    NoSuchAlgorithmException, NoSuchProviderException {
	return sha1Util.getFileHash(file);
    }

    /**
     * Gets the file hash as an hex string value.
     * 
     * @param file the file to hash
     * 
     * @return the file hash as an hex string
     * 
     * @exception IOException
     *                an I/O error occurred
     * @exception NoSuchAlgorithmException
     *                hash algorithm not found
     * @exception NoSuchProviderException
     *                hash provider not found
     */

    public String getHexFileHash(File file) throws IOException,
	    NoSuchAlgorithmException, NoSuchProviderException {
	return sha1Util.getHexFileHash(file);
    }

}
