/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//22/12/12 20:25 NDP : TomcatEmbeddedStore: no more references to AwakeDataSource

package org.awakefw.commons.server.util.embed;

import javax.sql.DataSource;

/**
 * Class that store static values for Tomcat Embedded Server. The values indicate if we are in EmbedMode and
 * the parameters to pass to Tomcat (Servlet, configurators, ...) at startup.
 * 
 * @author Nicolas de Pomereu
 *
 */
public class TomcatEmbeddedStore {

    /** Value that says we are in stand alone Awake Server with Tomcat Embed */
    private static boolean tomcatEmbedded = false;
    
    /** Says if The AwakeSQLManager status is ok after get call */
    private static boolean AwakeSQLManagerStatusOk = false;

    // Configurator class names - properties extract from file passed to AwakeServerMain
    private static String awakeCommonsConfiguratorClassName = null;
    private static String awakeSqlConfiguratorClassName = null;    
    private static String awakeFileConfiguratorClassName = null;
    private static String awakeFileActionManagerClassName = null;
        
    /** Awake SQL EE License info */
    private static String awakeLicenseKey = null;    
    private static String email = null;
    private static String name = null;
    private static String company = null;
    private static String siteCompany = null;
    
    /** Store a DataSource */
    private static DataSource dataSource = null;
        
    /**
     * no instantiation
     */
    private TomcatEmbeddedStore() {

    }

    /**
     * @return the tomcatEmbedded
     */
    public static boolean isTomcatEmbedded() {
        return tomcatEmbedded;
    }

    /**
     * @param tomcatEmbedded the tomcatEmbedded to set
     */
    public static void setTomcatEmbedded(boolean tomcatEmbedded) {
        TomcatEmbeddedStore.tomcatEmbedded = tomcatEmbedded;
    }

    /**
     * @return the awakeCommonsConfiguratorClassName
     */
    public static String getAwakeCommonsConfiguratorClassName() {
        return awakeCommonsConfiguratorClassName;
    }

    /**
     * @return the awakeSQLManagerStatusOk
     */
    public static boolean isAwakeSQLManagerStatusOk() {
        return AwakeSQLManagerStatusOk;
    }

    /**
     * @param awakeSQLManagerStatusOk the awakeSQLManagerStatusOk to set
     */
    public static void setAwakeSQLManagerStatusOk(boolean awakeSQLManagerStatusOk) {
        AwakeSQLManagerStatusOk = awakeSQLManagerStatusOk;
    }

    
    /**
     * @param awakeCommonsConfiguratorClassName the awakeCommonsConfiguratorClassName to set
     */
    public static void setAwakeCommonsConfiguratorClassName(
    	String awakeCommonsConfiguratorClassName) {
        TomcatEmbeddedStore.awakeCommonsConfiguratorClassName = awakeCommonsConfiguratorClassName;
    }

    /**
     * @return the awakeSqlConfiguratorClassName
     */
    public static String getAwakeSqlConfiguratorClassName() {
        return awakeSqlConfiguratorClassName;
    }

    /**
     * @param awakeSqlConfiguratorClassName the awakeSqlConfiguratorClassName to set
     */
    public static void setAwakeSqlConfiguratorClassName(
    	String awakeSqlConfiguratorClassName) {
        TomcatEmbeddedStore.awakeSqlConfiguratorClassName = awakeSqlConfiguratorClassName;
    }

    /**
     * @return the awakeFileConfiguratorClassName
     */
    public static String getAwakeFileConfiguratorClassName() {
        return awakeFileConfiguratorClassName;
    }

    /**
     * @param awakeFileConfiguratorClassName the awakeFileConfiguratorClassName to set
     */
    public static void setAwakeFileConfiguratorClassName(
    	String awakeFileConfiguratorClassName) {
        TomcatEmbeddedStore.awakeFileConfiguratorClassName = awakeFileConfiguratorClassName;
    }

    /**
     * @return the awakeFileActionManagerClassName
     */
    public static String getAwakeFileActionManagerClassName() {
        return awakeFileActionManagerClassName;
    }

    /**
     * @param awakeFileActionManagerClassName the awakeFileActionManagerClassName to set
     */
    public static void setAwakeFileActionManagerClassName(
    	String awakeFileActionManagerClassName) {
        TomcatEmbeddedStore.awakeFileActionManagerClassName = awakeFileActionManagerClassName;
    }

    /**
     * @return the dataSource
     */
    public static DataSource getDataSource() {
        return dataSource;
    }

    /**
     * @param dataSource the dataSource to set
     */
    public static void setDataSource(DataSource dataSource) {
        TomcatEmbeddedStore.dataSource = dataSource;
    }

    /**
     * @return the awakeLicenseKey
     */
    public static String getAwakeLicenseKey() {
        return awakeLicenseKey;
    }

    /**
     * @param awakeLicenseKey the awakeLicenseKey to set
     */
    public static void setAwakeLicenseKey(String awakeLicenseKey) {
        TomcatEmbeddedStore.awakeLicenseKey = awakeLicenseKey;
    }

    /**
     * @return the email
     */
    public static String getEmail() {
        return email;
    }

    /**
     * @param email the email to set
     */
    public static void setEmail(String email) {
        TomcatEmbeddedStore.email = email;
    }

    /**
     * @return the name
     */
    public static String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public static void setName(String name) {
        TomcatEmbeddedStore.name = name;
    }

    /**
     * @return the company
     */
    public static String getCompany() {
        return company;
    }

    /**
     * @param company the company to set
     */
    public static void setCompany(String company) {
        TomcatEmbeddedStore.company = company;
    }

    /**
     * @return the siteCompany
     */
    public static String getSiteCompany() {
        return siteCompany;
    }

    /**
     * @param siteCompany the siteCompany to set
     */
    public static void setSiteCompany(String siteCompany) {
        TomcatEmbeddedStore.siteCompany = siteCompany;
    }
                
}
