/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2012, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//Last Updates: 
// 27 aot 2010 18:55:49 Nicolas de Pomereu
// 15/03/12 12:15 NDP : ListOfStringTransport: Creation from JsonClientBuilder & JsonServerBuilder

package org.awakefw.file.json;

import java.util.List;
import java.util.Vector;

import org.awakefw.file.api.util.AwakeDebug;
import org.awakefw.file.util.AwakeClientLogger;
import org.json.simple.JSONArray;
import org.json.simple.JSONValue;

/**
 * @author Nicolas de Pomereu
 * 
 *         Class to transport a list of strings with JSON from the PC side.
 *         Because obsfucation creates toubles, this class must never obsfucated
 */

public class ListOfStringTransport {

    /** For debug info */
    private static boolean DEBUG = AwakeDebug.isSet(ListOfStringTransport.class);

    /**
     * Format for JSON String a list of strings
     * 
     * @param list
     *            a list of strings
     * 
     * @return the formated JSON string ready for transport
     */

    public static String toJson(List<String> list ) {

	if (list == null) {
	    throw new IllegalArgumentException("list is null!");
	}
		
	String jsonString = JSONValue.toJSONString(list);
	// free resultSetLine
	list = null;
	return jsonString;
	
    }


    /**
     * Format from JSON string a list of strings - TO BE USED ONLY ON THE SERVER
     * SIDE;
     * 
     * @param jsonString
     *            formated JSON string containing a list of strings
     * 
     * @return a list of strings
     */
    public static List<String> fromJson(String jsonString) {
        if (jsonString == null) {
            throw new IllegalArgumentException("jsonString is null!");
        }
    
        debug("List jsonString: " + jsonString);
            
	Object obj =JSONValue.parse(jsonString);
	JSONArray jsonArray=(JSONArray)obj;
	
	List<String> values = new Vector<String>();
	
	for (int i = 0; i < jsonArray.size(); i++) {
	    values.add((String) jsonArray.get(i));
	}
	
	// free JSONArray
	obj = null;
	jsonArray = null;
	
	
	return values;
	
    }

    private static void debug(String s) {
	if (DEBUG) {
	    AwakeClientLogger.log(s);
	}
    }

    
}
